@@ -835,6 +835,8 @@ ngx_http_test_expect(ngx_http_request_t *r)
 
     /* we assume that such small packet should be send successfully */
 
+    r->connection->error = 1;
+
     return NGX_ERROR;
 }
 
@@ -1085,6 +1087,7 @@ ngx_http_request_body_save_filter(ngx_http_request_t *r, ngx_chain_t *in)
 
 #if (NGX_DEBUG)
 
+#if 0
     for (cl = rb->bufs; cl; cl = cl->next) {
         ngx_log_debug7(NGX_LOG_DEBUG_EVENT, r->connection->log, 0,
                        "http body old buf t:%d f:%d %p, pos %p, size: %z "
@@ -1095,6 +1098,7 @@ ngx_http_request_body_save_filter(ngx_http_request_t *r, ngx_chain_t *in)
                        cl->buf->file_pos,
                        cl->buf->file_last - cl->buf->file_pos);
     }
+#endif
 
     for (cl = in; cl; cl = cl->next) {
         ngx_log_debug7(NGX_LOG_DEBUG_EVENT, r->connection->log, 0,
